# File: WeatherForecast.py
#
# Copyright (c) 2006 by ['Ben Hirsch']
#
# Berkeley Software Distribution License (BSD)
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#
#   * Neither the name of the Plone Foundation nor the names of its
#     contributors may be used to endorse or promote products derived
#     from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__author__ = """Ben Hirsch <benh@usa.net>"""
__docformat__ = 'plaintext'

import datetime
import xml.dom.minidom
import urllib

def getText(nodelist):
	rc = ""
	for node in nodelist:
		if node.nodeType == node.TEXT_NODE:
			rc = rc + node.data
	return rc

def getForecast(latitude=42.64645, longitude=-71.31418, days=5):
	# http://www.weather.gov/forecasts/xml/SOAP_server/ndfdSOAPclientByDay.php?lat=42.612635&lon=-71.320400&format=12+hourly&startDate=2006-07-14&numDays=4
	forecastURLbase = "http://www.weather.gov/forecasts/xml/SOAP_server/ndfdSOAPclientByDay.php?lat=%f&lon=%f&format=12+hourly&startDate=%s&numDays=%d"
	Forecast = {}
	daysList = []
	url = forecastURLbase % (
		latitude,
		longitude,
		datetime.date.today(),
		days)

	dom = xml.dom.minidom.parse(urllib.urlopen(url))
	
	layoutKeys = []
	days = {}
	temperatures = {}
	precip = []
	weather = []
	icons = []

	for timeLayout in dom.getElementsByTagName("time-layout"):
		layoutKey = getText(timeLayout.getElementsByTagName("layout-key")[0].childNodes)
		layoutKeys.append(layoutKey)
		days[layoutKey] = []
		for day in timeLayout.getElementsByTagName("start-valid-time"):
			days[layoutKey].append( day.getAttribute('period-name') )
			temperatures[layoutKey] = {}

	parameters = dom.getElementsByTagName("parameters")[0]

	for temperatureEl in parameters.getElementsByTagName("temperature"):
		layout = temperatureEl.getAttribute("time-layout")
		temperatures[layout] = []
		for value in temperatureEl.getElementsByTagName("value"):
			temperatures[layout].append( getText(value.childNodes) )

	for precipEl in parameters.getElementsByTagName("probability-of-precipitation"):
		for value in precipEl.getElementsByTagName("value"):
			precip.append( getText(value.childNodes) )

	for weatherEl in parameters.getElementsByTagName("weather"):
		for value in weatherEl.getElementsByTagName("weather-conditions"):
			weather.append( value.getAttribute("weather-summary") )

	for iconEl in parameters.getElementsByTagName("conditions-icon"):
		for value in iconEl.getElementsByTagName("icon-link"):
			icons.append( getText(value.childNodes) )

	daysList = days[layoutKeys[2]]
	for x in range( 0, len(daysList), 1 ):
		curDay = daysList[x]
		Forecast[curDay] = {}
		if x % 2 == 0:
			Forecast[curDay]["Temp"] = temperatures[layoutKeys[0]][x/2]
		else:
			Forecast[curDay]["Temp"] = temperatures[layoutKeys[1]][x/2]
		Forecast[curDay]["Precip"] = precip[x]
		Forecast[curDay]["Weather"] = weather[x]
		Forecast[curDay]["Icon"] = icons[x]

	return (daysList, Forecast)
